@@ -49,7 +49,7 @@ def _check_valid_version():
     '''
     # pylint: disable=no-member
     npm_version = distutils.version.LooseVersion(
-        salt.modules.cmdmod.run('npm --version', python_shell=True))
+        salt.modules.cmdmod.run('npm --version', output_loglevel='quiet'))
     valid_version = distutils.version.LooseVersion('1.2')
     # pylint: enable=no-member
     if npm_version < valid_version:
@@ -65,7 +65,9 @@ def install(pkg=None,
             dir=None,
             runas=None,
             registry=None,
-            env=None):
+            env=None,
+            dry_run=False,
+            silent=True):
     '''
     Install an NPM package.
 
@@ -101,6 +103,21 @@ def install(pkg=None,
 
         .. versionadded:: 2014.7.0
 
+    silent
+        Whether or not to run NPM install with --silent flag.
+
+        .. versionadded:: 2016.3.0
+
+    dry_run
+        Whether or not to run NPM install with --dry-run flag.
+
+        .. versionadded:: 2015.8.4
+
+    silent
+        Whether or not to run NPM install with --silent flag.
+
+        .. versionadded:: 2015.8.5
+
     CLI Example:
 
     .. code-block:: bash
@@ -112,32 +129,30 @@ def install(pkg=None,
     '''
     # Protect against injection
     if pkg:
-        pkg = _cmd_quote(pkg)
-    if pkgs:
-        pkg_list = []
-        for item in pkgs:
-            pkg_list.append(_cmd_quote(item))
-        pkgs = pkg_list
+        pkgs = [_cmd_quote(pkg)]
+    elif pkgs:
+        pkgs = [_cmd_quote(v) for v in pkgs]
+    else:
+        pkgs = []
     if registry:
         registry = _cmd_quote(registry)
 
-    cmd = ['npm', 'install', '--silent', '--json']
+    cmd = ['npm', 'install', '--json']
+    if silent:
+        cmd.append('--silent')
 
-    if dir is None:
-        cmd.append(' --global')
+    if not dir:
+        cmd.append('--global')
 
     if registry:
-        cmd.append(' --registry="{0}"'.format(registry))
+        cmd.append('--registry="{0}"'.format(registry))
 
-    if pkg:
-        cmd.append(pkg)
-    elif pkgs:
-        cmd.extend(pkgs)
-    else:
-        return 'No package name specified'
+    if dry_run:
+        cmd.append('--dry-run')
 
-    if env is None:
-        env = {}
+    cmd.extend(pkgs)
+
+    env = env or {}
 
     if runas:
         uid = salt.utils.get_uid(runas)
@@ -155,30 +170,34 @@ def install(pkg=None,
     try:
         return json.loads(npm_output)
     except ValueError:
-        # Not JSON! Try to coax the json out of it!
         pass
 
+    json_npm_output = _extract_json(npm_output)
+    return json_npm_output or npm_output
+
+
+def _extract_json(npm_output):
     lines = npm_output.splitlines()
     log.error(lines)
 
-    while lines:
-        # Strip all lines until JSON output starts
-        while not lines[0].startswith('{') and not lines[0].startswith('['):
-            lines = lines[1:]
-
-        try:
-            return json.loads(''.join(lines))
-        except ValueError:
-            lines = lines[1:]
-
-    # Still no JSON!! Return the stdout as a string
-    return npm_output
+    # Strip all lines until JSON output starts
+    while lines and not lines[0].startswith('{') and not lines[0].startswith('['):
+        lines = lines[1:]
+    while lines and not lines[-1].startswith('}') and not lines[-1].startswith(']'):
+        lines = lines[:-1]
+    # macOS with fsevents includes the following line in the return
+    # when a new module is installed which is invalid JSON:
+    #     [fsevents] Success: "..."
+    while lines and (lines[0].startswith('[fsevents]') or lines[0].startswith('Pass ')):
+        lines = lines[1:]
+    try:
+        return json.loads(''.join(lines))
+    except ValueError:
+        pass
+    return None
 
 
-def uninstall(pkg,
-              dir=None,
-              runas=None,
-              env=None):
+def uninstall(pkg, dir=None, runas=None, env=None):
     '''
     Uninstall an NPM package.
 
@@ -212,20 +231,18 @@ def uninstall(pkg,
     if pkg:
         pkg = _cmd_quote(pkg)
 
-    if env is None:
-        env = {}
+    env = env or {}
 
     if runas:
         uid = salt.utils.get_uid(runas)
         if uid:
             env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
 
-    cmd = 'npm uninstall'
+    cmd = ['npm', 'uninstall', '"{0}"'.format(pkg)]
+    if not dir:
+        cmd.append('--global')
 
-    if dir is None:
-        cmd += ' --global'
-
-    cmd += ' "{0}"'.format(pkg)
+    cmd = ' '.join(cmd)
 
     result = __salt__['cmd.run_all'](cmd, python_shell=True, cwd=dir, runas=runas, env=env)
 
@@ -235,10 +252,7 @@ def uninstall(pkg,
     return True
 
 
-def list_(pkg=None,
-            dir=None,
-            runas=None,
-            env=None):
+def list_(pkg=None, dir=None, runas=None, env=None, depth=None):
     '''
     List installed NPM packages.
 
@@ -264,6 +278,11 @@ def list_(pkg=None,
 
         .. versionadded:: 2014.7.0
 
+    depth
+        Limit the depth of the packages listed
+
+        .. versionadded:: 2016.11.6, Nitrogen
+
     CLI Example:
 
     .. code-block:: bash
@@ -271,33 +290,31 @@ def list_(pkg=None,
         salt '*' npm.list
 
     '''
-    # Protect against injection
-    if pkg:
-        pkg = _cmd_quote(pkg)
-
-    if env is None:
-        env = {}
+    env = env or {}
 
     if runas:
         uid = salt.utils.get_uid(runas)
         if uid:
             env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
 
-    cmd = 'npm list --silent --json'
+    cmd = ['npm', 'list', '--json', '--silent']
 
-    if dir is None:
-        cmd += ' --global'
+    if not dir:
+        cmd.append('--global')
+
+    if depth is not None:
+        if not isinstance(depth, (int, float)):
+            raise salt.exceptions.SaltInvocationError('Error: depth {0} must be a number'.format(depth))
+        cmd.append('--depth={0}'.format(int(depth)))
 
     if pkg:
-        cmd += ' "{0}"'.format(pkg)
+        # Protect against injection
+        pkg = _cmd_quote(pkg)
+        cmd.append('"{0}"'.format(pkg))
+    cmd = ' '.join(cmd)
 
     result = __salt__['cmd.run_all'](
-            cmd,
-            cwd=dir,
-            runas=runas,
-            env=env,
-            python_shell=True,
-            ignore_retcode=True)
+        cmd, cwd=dir, runas=runas, env=env, python_shell=True, ignore_retcode=True)
 
     # npm will return error code 1 for both no packages found and an actual
     # error. The only difference between the two cases are if stderr is empty
@@ -305,3 +322,134 @@ def list_(pkg=None,
         raise CommandExecutionError(result['stderr'])
 
     return json.loads(result['stdout']).get('dependencies', {})
+
+
+def cache_clean(path=None, runas=None, env=None, force=False):
+    '''
+    Clean cached NPM packages.
+
+    If no path for a specific package is provided the entire cache will be cleared.
+
+    path
+        The cache subpath to delete, or None to clear the entire cache
+
+    runas
+        The user to run NPM with
+
+    env
+        Environment variables to set when invoking npm. Uses the same ``env``
+        format as the :py:func:`cmd.run <salt.modules.cmdmod.run>` execution
+        function.
+
+    force
+        Force cleaning of cache.  Required for npm@5 and greater
+
+        .. versionadded:: 2016.11.6
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' npm.cache_clean force=True
+
+    '''
+    env = env or {}
+
+    if runas:
+        uid = salt.utils.get_uid(runas)
+        if uid:
+            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+
+    cmd = ['npm', 'cache', 'clean']
+    if path:
+        cmd.append(path)
+    if force is True:
+        cmd.append('--force')
+
+    cmd = ' '.join(cmd)
+    result = __salt__['cmd.run_all'](
+        cmd, cwd=None, runas=runas, env=env, python_shell=True, ignore_retcode=True)
+
+    if result['retcode'] != 0:
+        log.error(result['stderr'])
+        return False
+    return True
+
+
+def cache_list(path=None, runas=None, env=None):
+    '''
+    List NPM cached packages.
+
+    If no path for a specific package is provided this will list all the cached packages.
+
+    path
+        The cache subpath to list, or None to list the entire cache
+
+    runas
+        The user to run NPM with
+
+    env
+        Environment variables to set when invoking npm. Uses the same ``env``
+        format as the :py:func:`cmd.run <salt.modules.cmdmod.run>` execution
+        function.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' npm.cache_clean
+
+    '''
+    env = env or {}
+
+    if runas:
+        uid = salt.utils.get_uid(runas)
+        if uid:
+            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+
+    cmd = ['npm', 'cache', 'ls']
+    if path:
+        cmd.append(path)
+
+    cmd = ' '.join(cmd)
+    result = __salt__['cmd.run_all'](
+        cmd, cwd=None, runas=runas, env=env, python_shell=True, ignore_retcode=True)
+
+    if result['retcode'] != 0 and result['stderr']:
+        raise CommandExecutionError(result['stderr'])
+
+    return result['stdout']
+
+
+def cache_path(runas=None, env=None):
+    '''
+    List path of the NPM cache directory.
+
+    runas
+        The user to run NPM with
+
+    env
+        Environment variables to set when invoking npm. Uses the same ``env``
+        format as the :py:func:`cmd.run <salt.modules.cmdmod.run>` execution
+        function.
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' npm.cache_path
+
+    '''
+    env = env or {}
+
+    if runas:
+        uid = salt.utils.get_uid(runas)
+        if uid:
+            env.update({'SUDO_UID': b'{0}'.format(uid), 'SUDO_USER': b''})
+
+    cmd = 'npm config get cache'
+
+    result = __salt__['cmd.run_all'](
+        cmd, cwd=None, runas=runas, env=env, python_shell=True, ignore_retcode=True)
+
+    return result.get('stdout') or result.get('stderr')
