@@ -11,15 +11,20 @@ Package support for pkgin based systems, inspired from freebsdpkg module
 
 # Import python libs
 from __future__ import absolute_import
+import copy
+import logging
 import os
 import re
-import logging
 
 # Import salt libs
 import salt.utils
+import salt.utils.pkg
 import salt.utils.decorators as decorators
 from salt.exceptions import CommandExecutionError, MinionError
 
+# Import 3rd-party libs
+import salt.ext.six as six
+
 VERSION_MATCH = re.compile(r'pkgin(?:[\s]+)([\d.]+)(?:[\s]+)(?:.*)')
 log = logging.getLogger(__name__)
 
@@ -35,15 +40,17 @@ def _check_pkgin():
     ppath = salt.utils.which('pkgin')
     if ppath is None:
         # pkgin was not found in $PATH, try to find it via LOCALBASE
-        localbase = __salt__['cmd.run'](
-            'pkg_info -Q LOCALBASE pkgin',
-            output_loglevel='trace'
-        )
-        if localbase is not None:
-            ppath = '{0}/bin/pkgin'.format(localbase)
-            if not os.path.exists(ppath):
-                return None
-
+        try:
+            localbase = __salt__['cmd.run'](
+               'pkg_info -Q LOCALBASE pkgin',
+                output_loglevel='trace'
+            )
+            if localbase is not None:
+                ppath = '{0}/bin/pkgin'.format(localbase)
+                if not os.path.exists(ppath):
+                    return None
+        except CommandExecutionError:
+            return None
     return ppath
 
 
@@ -76,15 +83,6 @@ def _supports_regex():
     return tuple([int(i) for i in _get_version()]) > (0, 5)
 
 
-@decorators.memoize
-def _supports_parsing():
-    '''
-    Check support of parsing
-    '''
-
-    return tuple([int(i) for i in _get_version()]) > (0, 7)
-
-
 def __virtual__():
     '''
     Set the virtual pkg module if the os is supported by pkgin
@@ -93,7 +91,8 @@ def __virtual__():
 
     if __grains__['os'] in supported and _check_pkgin():
         return __virtualname__
-    return False
+    return (False, 'The pkgin execution module cannot be loaded: only '
+            'available on {0} systems.'.format(', '.join(supported)))
 
 
 def _splitpkg(name):
@@ -136,6 +135,7 @@ def search(pkg_name):
 
 def latest_version(*names, **kwargs):
     '''
+    .. versionchanged: 2016.3.0
     Return the latest version of the named package available for upgrade or
     installation.
 
@@ -169,17 +169,22 @@ def latest_version(*names, **kwargs):
             output_loglevel='trace'
         )
         for line in out.splitlines():
-            p = line.split()  # pkgname-version status
-            if p and p[0] in ('=:', '<:', '>:'):
+            if _supports_regex():  # split on ;
+                p = line.split(';')
+            else:
+                p = line.split()  # pkgname-version status
+
+            if p and p[0] in ('=:', '<:', '>:', ''):
                 # These are explanation comments
                 continue
             elif p:
                 s = _splitpkg(p[0])
                 if s:
-                    if len(p) > 1 and p[1] == '<':
-                        pkglist[s[0]] = s[1]
-                    else:
-                        pkglist[s[0]] = ''
+                    if not s[0] in pkglist:
+                        if len(p) > 1 and p[1] == '<':
+                            pkglist[s[0]] = s[1]
+                        else:
+                            pkglist[s[0]] = ''
 
     if len(names) == 1 and pkglist:
         return pkglist[names[0]]
@@ -188,7 +193,7 @@ def latest_version(*names, **kwargs):
 
 
 # available_version is being deprecated
-available_version = latest_version
+available_version = salt.utils.alias_function(latest_version, 'available_version')
 
 
 def version(*names, **kwargs):
@@ -217,7 +222,8 @@ def refresh_db():
 
         salt '*' pkg.refresh_db
     '''
-
+    # Remove rtag file to keep multiple refreshes from happening in pkg states
+    salt.utils.pkg.clear_rtag(__opts__)
     pkgin = _check_pkgin()
 
     if pkgin:
@@ -232,11 +238,12 @@ def refresh_db():
                 '{0}'.format(comment)
             )
 
-    return {}
+    return True
 
 
 def list_pkgs(versions_as_list=False, **kwargs):
     '''
+    .. versionchanged: 2016.3.0
     List the packages currently installed as a dict::
 
         {'<package_name>': '<version>'}
@@ -253,6 +260,14 @@ def list_pkgs(versions_as_list=False, **kwargs):
             for x in ('removed', 'purge_desired')]):
         return {}
 
+    if 'pkg.list_pkgs' in __context__:
+        if versions_as_list:
+            return __context__['pkg.list_pkgs']
+        else:
+            ret = copy.deepcopy(__context__['pkg.list_pkgs'])
+            __salt__['pkg_resource.stringify'](ret)
+            return ret
+
     pkgin = _check_pkgin()
     if pkgin:
         pkg_command = '{0} ls'.format(pkgin)
@@ -264,15 +279,15 @@ def list_pkgs(versions_as_list=False, **kwargs):
     out = __salt__['cmd.run'](pkg_command, output_loglevel='trace')
     for line in out.splitlines():
         try:
-            if _supports_parsing():
-                pkg, ver = line.split(';', 1)[0].rsplit('-', 1)
-            else:
-                pkg, ver = line.split(' ', 1)[0].rsplit('-', 1)
+            # Some versions of pkgin check isatty unfortunately
+            # this results in cases where a ' ' or ';' can be used
+            pkg, ver = re.split('[; ]', line, 1)[0].rsplit('-', 1)
         except ValueError:
             continue
         __salt__['pkg_resource.add_pkg'](ret, pkg, ver)
 
     __salt__['pkg_resource.sort_pkglist'](ret)
+    __context__['pkg.list_pkgs'] = copy.deepcopy(ret)
     if not versions_as_list:
         __salt__['pkg_resource.stringify'](ret)
     return ret
@@ -367,25 +382,43 @@ def install(name=None, refresh=False, fromrepo=None,
     args.extend(pkg_params)
 
     old = list_pkgs()
-    __salt__['cmd.run'](
+
+    out = __salt__['cmd.run_all'](
         '{0} {1}'.format(cmd, ' '.join(args)),
         env=env,
         output_loglevel='trace'
     )
+
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
+    __context__.pop('pkg.list_pkgs', None)
     new = list_pkgs()
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered installing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
 
     _rehash()
-    return salt.utils.compare_dicts(old, new)
+    return ret
 
 
 def upgrade():
     '''
     Run pkg upgrade, if pkgin used. Otherwise do nothing
 
-    Return a dict containing the new package names and versions::
+    Returns a dictionary containing the changes:
+
+    .. code-block:: python
+
+        {'<package>':  {'old': '<old-version>',
+                        'new': '<new-version>'}}
 
-        {'<package>': {'old': '<old-version>',
-                       'new': '<new-version>'}}
 
     CLI Example:
 
@@ -393,28 +426,27 @@ def upgrade():
 
         salt '*' pkg.upgrade
     '''
-    ret = {'changes': {},
-           'result': True,
-           'comment': '',
-           }
-
     pkgin = _check_pkgin()
     if not pkgin:
         # There is not easy way to upgrade packages with old package system
         return {}
 
     old = list_pkgs()
-    call = __salt__['cmd.run_all']('{0} -y fug'.format(pkgin))
-    if call['retcode'] != 0:
-        ret['result'] = False
-        if 'stderr' in call:
-            ret['comment'] += call['stderr']
-        if 'stdout' in call:
-            ret['comment'] += call['stdout']
-    else:
-        __context__.pop('pkg.list_pkgs', None)
-        new = list_pkgs()
-        ret['changes'] = salt.utils.compare_dicts(old, new)
+
+    cmd = [pkgin, '-y', 'fug']
+    result = __salt__['cmd.run_all'](cmd,
+                                     output_loglevel='trace',
+                                     python_shell=False)
+    __context__.pop('pkg.list_pkgs', None)
+    new = list_pkgs()
+    ret = salt.utils.compare_dicts(old, new)
+
+    if result['retcode'] != 0:
+        raise CommandExecutionError(
+            'Problem encountered upgrading packages',
+            info={'changes': ret, 'result': result}
+        )
+
     return ret
 
 
@@ -476,10 +508,27 @@ def remove(name=None, pkgs=None, **kwargs):
     else:
         cmd = 'pkg_remove {0}'.format(for_remove)
 
-    __salt__['cmd.run'](cmd, output_loglevel='trace')
-    new = list_pkgs()
+    out = __salt__['cmd.run_all'](
+        cmd,
+        output_loglevel='trace'
+    )
 
-    return salt.utils.compare_dicts(old, new)
+    if out['retcode'] != 0 and out['stderr']:
+        errors = [out['stderr']]
+    else:
+        errors = []
+
+    __context__.pop('pkg.list_pkgs', None)
+    new = list_pkgs()
+    ret = salt.utils.compare_dicts(old, new)
+
+    if errors:
+        raise CommandExecutionError(
+            'Problem encountered removing package(s)',
+            info={'errors': errors, 'changes': ret}
+        )
+
+    return ret
 
 
 def purge(name=None, pkgs=None, **kwargs):
@@ -536,41 +585,45 @@ def file_list(package):
     '''
     ret = file_dict(package)
     files = []
-    for pkg_files in ret['files'].values():
+    for pkg_files in six.itervalues(ret['files']):
         files.extend(pkg_files)
     ret['files'] = files
     return ret
 
 
-def file_dict(package):
+def file_dict(*packages):
     '''
+    .. versionchanged: 2016.3.0
     List the files that belong to a package.
 
     CLI Examples:
 
     .. code-block:: bash
 
-        salt '*' pkg.file_list nginx
+        salt '*' pkg.file_dict nginx
+        salt '*' pkg.file_dict nginx varnish
     '''
     errors = []
     files = {}
-    files[package] = None
 
-    cmd = 'pkg_info -qL {0}'.format(package)
-    ret = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
+    for package in packages:
+        cmd = 'pkg_info -qL {0}'.format(package)
+        ret = __salt__['cmd.run_all'](cmd, output_loglevel='trace')
 
-    for line in ret['stderr'].splitlines():
-        errors.append(line)
+        files[package] = []
+        for line in ret['stderr'].splitlines():
+            errors.append(line)
 
-    for line in ret['stdout'].splitlines():
-        if line.startswith('/'):
-            if files[package] is None:
-                files[package] = [line]
-            else:
+        for line in ret['stdout'].splitlines():
+            if line.startswith('/'):
                 files[package].append(line)
-        else:
-            continue  # unexpected string
+            else:
+                continue  # unexpected string
 
-    return {'errors': errors, 'files': files}
+    ret = {'errors': errors, 'files': files}
+    for field in ret.keys():
+        if not ret[field] or ret[field] == '':
+            del ret[field]
+    return ret
 
 # vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
