@@ -91,12 +91,11 @@ from __future__ import absolute_import
 import sys
 
 # Import salt libs
-import salt.utils
-
 from salt.exceptions import CommandExecutionError, SaltInvocationError
 from salt.modules.aptpkg import _strip_uri
 from salt.state import STATE_INTERNAL_KEYWORDS as _STATE_INTERNAL_KEYWORDS
 import salt.utils
+import salt.utils.pkg.deb
 
 
 def __virtual__():
@@ -138,7 +137,7 @@ def managed(name, ppa=None, **kwargs):
 
     humanname
         This is used as the "name" value in the repo file in
-        ``/etc/yum.repos.d/`` (or ``/etc/zypp/repos.d`` for Suse distros).
+        ``/etc/yum.repos.d/`` (or ``/etc/zypp/repos.d`` for SUSE distros).
 
     baseurl
         The URL to a yum repository
@@ -223,6 +222,11 @@ def managed(name, ppa=None, **kwargs):
         the reverse will be passed as ``disabled``. For example, passing
         ``enabled=False`` will assume ``disabled=False``.
 
+    architectures
+        On apt-based systems, architectures can restrict the available
+        architectures that the repository provides (e.g. only amd64).
+        architectures should be a comma-separated list.
+
     comps
         On apt-based systems, comps dictate the types of packages to be
         installed from the repository (e.g. main, nonfree, ...).  For
@@ -388,13 +392,16 @@ def managed(name, ppa=None, **kwargs):
                 # split the line and sort everything after the URL
                 sanitizedsplit = sanitizedkwargs[kwarg].split()
                 sanitizedsplit[3:] = sorted(sanitizedsplit[3:])
-                reposplit = pre[kwarg].split()
+                reposplit, _, pre_comments = \
+                    [x.strip() for x in pre[kwarg].partition('#')]
+                reposplit = reposplit.split()
                 reposplit[3:] = sorted(reposplit[3:])
                 if sanitizedsplit != reposplit:
                     break
                 if 'comments' in kwargs:
-                    _line = pre[kwarg].split('#')
-                    if str(kwargs['comments']) not in _line:
+                    post_comments = \
+                        salt.utils.pkg.deb.combine_comments(kwargs['comments'])
+                    if pre_comments != post_comments:
                         break
             else:
                 if os_family in ('redhat', 'suse') \
@@ -459,7 +466,7 @@ def managed(name, ppa=None, **kwargs):
     except Exception as exc:
         ret['result'] = False
         ret['comment'] = \
-            'Failed to confirm config of repo {0!r}: {1}'.format(name, exc)
+            'Failed to confirm config of repo \'{0}\': {1}'.format(name, exc)
 
     # Clear cache of available packages, if present, since changes to the
     # repositories may change the packages that are available.
@@ -544,7 +551,7 @@ def absent(name, **kwargs):
     except CommandExecutionError as exc:
         ret['result'] = False
         ret['comment'] = \
-            'Failed to configure repo {0!r}: {1}'.format(name, exc)
+            'Failed to configure repo \'{0}\': {1}'.format(name, exc)
         return ret
 
     if not repo:
@@ -553,7 +560,7 @@ def absent(name, **kwargs):
         return ret
 
     if __opts__['test']:
-        ret['comment'] = ('Package repo {0!r} will be removed. This may '
+        ret['comment'] = ('Package repo \'{0}\' will be removed. This may '
                           'cause pkg states to behave differently than stated '
                           'if this action is repeated without test=True, due '
                           'to the differences in the configured repositories.'
