@@ -1,8 +1,6 @@
 # -*- coding: utf-8 -*-
 '''
     :codeauthor: :email:`Pedro Algarvio (pedro@algarvio.me)`
-    :copyright: © 2012-2013 by the SaltStack Team, see AUTHORS for more details
-    :license: Apache 2.0, see LICENSE for more details.
 
 
     tests.integration.shell.cp
@@ -10,9 +8,13 @@
 '''
 
 # Import python libs
+from __future__ import absolute_import
+import errno
 import os
 import yaml
 import pipes
+import shutil
+import tempfile
 
 # Import Salt Testing libs
 from salttesting.helpers import ensure_in_syspath
@@ -22,6 +24,9 @@ ensure_in_syspath('../../')
 import integration
 import salt.utils
 
+# Import 3rd-party libs
+import salt.ext.six as six
+
 
 class CopyTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
 
@@ -46,7 +51,8 @@ class CopyTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
                 'files', 'file', 'base', 'testfile'
             )
         )
-        testfile_contents = salt.utils.fopen(testfile, 'r').read()
+        with salt.utils.fopen(testfile, 'r') as fh_:
+            testfile_contents = fh_.read()
 
         for idx, minion in enumerate(minions):
             ret = self.run_salt(
@@ -70,14 +76,14 @@ class CopyTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
                 integration.TMP, 'cp_{0}_testfile'.format(idx)
             )
 
-            ret = self.run_cp('{0} {1} {2}'.format(
+            ret = self.run_cp('--out pprint {0} {1} {2}'.format(
                 pipes.quote(minion),
                 pipes.quote(testfile),
                 pipes.quote(minion_testfile)
             ))
 
             data = yaml.load('\n'.join(ret))
-            for part in data.values():
+            for part in six.itervalues(data):
                 self.assertTrue(part[minion_testfile])
 
             ret = self.run_salt(
@@ -107,6 +113,63 @@ class CopyTest(integration.ShellCase, integration.ShellCaseCommonTestsMixIn):
             data = yaml.load('\n'.join(ret))
             self.assertTrue(data[minion])
 
+    def test_issue_7754(self):
+        config_dir = os.path.join(integration.TMP, 'issue-7754')
+
+        try:
+            os.makedirs(config_dir)
+        except OSError as exc:
+            if exc.errno != errno.EEXIST:
+                raise
+
+        config_file_name = 'master'
+        with salt.utils.fopen(self.get_config_file_path(config_file_name), 'r') as fhr:
+            config = yaml.load(fhr.read())
+            config['log_file'] = 'file:///dev/log/LOG_LOCAL3'
+            with salt.utils.fopen(os.path.join(config_dir, config_file_name), 'w') as fhw:
+                fhw.write(
+                    yaml.dump(config, default_flow_style=False)
+                )
+
+        try:
+            fd_, fn_ = tempfile.mkstemp()
+            os.close(fd_)
+
+            with salt.utils.fopen(fn_, 'w') as fp_:
+                fp_.write('Hello world!\n')
+
+            ret = self.run_script(
+                self._call_binary_,
+                '--out pprint --config-dir {0} \'*\' {1} {0}/{2}'.format(
+                    config_dir,
+                    fn_,
+                    os.path.basename(fn_),
+                ),
+                catch_stderr=True,
+                with_retcode=True
+            )
+
+            self.assertIn('minion', '\n'.join(ret[0]))
+            self.assertIn('sub_minion', '\n'.join(ret[0]))
+            self.assertFalse(os.path.isdir(os.path.join(config_dir, 'file:')))
+        except AssertionError:
+            if os.path.exists('/dev/log') and ret[2] != 2:
+                # If there's a syslog device and the exit code was not 2, 'No
+                # such file or directory', raise the error
+                raise
+            self.assertIn(
+                'Failed to setup the Syslog logging handler', '\n'.join(ret[1])
+            )
+            self.assertEqual(ret[2], 2)
+        finally:
+            try:
+                os.remove(fn_)
+            except OSError as exc:
+                if exc.errno != errno.ENOENT:
+                    raise
+            if os.path.isdir(config_dir):
+                shutil.rmtree(config_dir)
+
 
 if __name__ == '__main__':
     from integration import run_tests
