@@ -40,6 +40,10 @@ class GitTest(integration.ModuleCase, integration.SaltReturnAssertsMixIn):
             msg = 'error resolving {0}, possible network issue?'
             self.skipTest(msg.format(self.__domain))
 
+    def tearDown(self):
+        # Reset the dns timeout after the test is over
+        socket.setdefaulttimeout(None)
+
     def test_latest(self):
         '''
         git.latest
@@ -517,7 +521,7 @@ class LocalRepoGitTest(integration.ModuleCase, integration.SaltReturnAssertsMixI
             os.path.join(repo, 'refs', 'heads', 'develop')
         )
 
-        # Run git.latest state. This should successfuly clone and fail with a
+        # Run git.latest state. This should successfully clone and fail with a
         # specific error in the comment field.
         ret = self.run_state(
             'git.latest',
